// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate Lua-API
// File          : Lua-CANMessage.hpp
// Author        : Stefan Welisch
// Copyright     : (C) 2009 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id:$
//
// $Log:$
//
// ----------------------------------------------------------------------------

#ifndef LUA_CANOPEN_MESSAGE_HPP
#define LUA_CANOPEN_MESSAGE_HPP

// Includes -------------------------------------------------------------------
#include <sys/types.h>
#include <vector>

// Defines --------------------------------------------------------------------

// packing mode 1 Byte
//#pragma pack(1)

#ifndef USE_DEFAULT_PACKING
   // packing mode 1 Byte
#pragma pack(push,1)
#endif

/** Class implements a Lua-CANopen PDO message.
    @author  Stefan Welisch
    @version 1.0
*/
class CLuaCANopenPDOMessage
{
   public:
      enum { MAX_PDO_LENGTH  = 8 };
      
      CLuaCANopenPDOMessage( void ){};

      /** Constructor.
          @param nHandle       Handle from connection.
          @param nIdentifier   Telegram id.
          @param pcData        Pointer to data buffer.
          @param nDataLength   Length og data buffer.
          @param nFormatFlags  Format flags.
          @param nSeconds      Seconds.
          @param nMicroseconds Microseconds.
      */
      CLuaCANopenPDOMessage( const unsigned int    nHandle,
                             const unsigned int    nNodeID,
                             const unsigned char   nPDOTyp,
                             const unsigned char   nDataLength,
                             const unsigned char * pcData,
                             const unsigned int    nSeconds,
                             const unsigned int    nMicroseconds);

      /** Destructor */
      virtual ~CLuaCANopenPDOMessage(){};

   protected:
   public:
      unsigned int  m_nHandle;                      ///< Handle from connection
      unsigned int  m_nNodeID;                      ///< Message id
      unsigned char m_nPDOTyp;
      unsigned int  m_nSeconds;                     ///< Seconds of message receipt (since 1970)
      unsigned int  m_nMicroseconds;                ///< Microseconds of time

      unsigned char m_nDataLength;                  ///< Message length
      unsigned char m_acDataBuffer[MAX_PDO_LENGTH]; ///< Data buffer
};


class CLuaCANopenSYNCMessage
{
   public:
      CLuaCANopenSYNCMessage( void ){};

      /** Constructor.
          @param nHandle       Handle from connection.
          @param nIdentifier   Telegram id.
          @param pcData        Pointer to data buffer.
          @param nDataLength   Length og data buffer.
          @param nFormatFlags  Format flags.
          @param nSeconds      Seconds.
          @param nMicroseconds Microseconds.
      */
      CLuaCANopenSYNCMessage( const unsigned int  nHandle,
                              const unsigned char nReturnCode,
                              const unsigned int  nSeconds,
                              const unsigned int  nMicroseconds);
 
      /** Destructor */
      virtual ~CLuaCANopenSYNCMessage(){};

   protected:
   public:
      unsigned int   m_nHandle;                      ///< Handle from connection
      unsigned char  m_nReturnCode;
      unsigned int   m_nSeconds;                     ///< Seconds of message receipt (since 1970)
      unsigned int   m_nMicroseconds;                ///< Microseconds of time
};


class CLuaCANopenEMCYMessage
{
   public:
      CLuaCANopenEMCYMessage( void ){};

      /** Constructor.
          @param nHandle       Handle from connection.
          @param nIdentifier   Telegram id.
          @param pcData        Pointer to data buffer.
          @param nDataLength   Length og data buffer.
          @param nFormatFlags  Format flags.
          @param nSeconds      Seconds.
          @param nMicroseconds Microseconds.
      */
      CLuaCANopenEMCYMessage( const unsigned int   nHandle,
                              const unsigned char  nNodeID,
                              const unsigned short nErrorCode,
                              const unsigned char  nErrorRegister,
                              const unsigned char  nErrorDescription[5],
                              const unsigned int   nSeconds,
                              const unsigned int   nMicroseconds);

      /** Destructor */
      virtual ~CLuaCANopenEMCYMessage(){};

   protected:
   public:
      unsigned int   m_nHandle;                      ///< Handle from connection
      unsigned char  m_nNodeID;
      unsigned short m_nErrorCode;
      unsigned char  m_nErrorRegister;
      unsigned char  m_nErrorDescription[5];
      unsigned int   m_nSeconds;                     ///< Seconds of message receipt (since 1970)
      unsigned int   m_nMicroseconds;                ///< Microseconds of time
};



class CLuaCANopenGUARDMessage
{
   public:
      CLuaCANopenGUARDMessage( void ){};

      /** Constructor.
          @param nHandle       Handle from connection.
          @param nNodeID       NodeID.
          @param nStatus       Status of CANopen slave.
          @param nSeconds      Seconds.
          @param nMicroseconds Microseconds.
      */
      CLuaCANopenGUARDMessage( const unsigned int  nHandle,
                               const unsigned char nNodeID,
                               const unsigned char nStatus,
                               const unsigned int  nSeconds,
                               const unsigned int  nMicroseconds);

      /** Destructor */
      virtual ~CLuaCANopenGUARDMessage(){};

   protected:
   public:
      unsigned int  m_nHandle;                      ///< Handle from connection
      unsigned char m_nNodeID;
      unsigned char m_nStatus;
      unsigned int  m_nSeconds;                     ///< Seconds of message receipt (since 1970)
      unsigned int  m_nMicroseconds;                ///< Microseconds of time
};


/** Class implements a Lua-CANopen Undefined message.
    @author  Stefan Welisch
    @version 1.0
*/
class CLuaCANopenUndefinedMessage
{
   public:
      enum { MAX_UNDEFINED_LENGTH  = 8 };
      
      CLuaCANopenUndefinedMessage( void ){};

      /** Constructor.
          @param nHandle       Handle from connection.
          @param nIdentifier   Telegram id.
          @param pcData        Pointer to data buffer.
          @param nDataLength   Length og data buffer.
          @param nFormatFlags  Format flags.
          @param nSeconds      Seconds.
          @param nMicroseconds Microseconds.
      */
      CLuaCANopenUndefinedMessage( const unsigned int    nHandle,
                                   const unsigned int    nCANID,
                                   const unsigned char   nNodeID,
                                   const unsigned char   nFunctionCode,
                                   const unsigned char   nDataLength,
                                   const unsigned char * pcData,
                                   const unsigned int    nSeconds,
                                   const unsigned int    nMicroseconds);

      /** Destructor */
      virtual ~CLuaCANopenUndefinedMessage(){};

   protected:
   public:
      unsigned int  m_nHandle;                      ///< Handle from connection
      unsigned int  m_nCANID;                       ///< CAN ID
      unsigned char m_nNodeID;                      ///< Node ID
      unsigned char m_nFunctionCode;
      unsigned int  m_nSeconds;                     ///< Seconds of message receipt (since 1970)
      unsigned int  m_nMicroseconds;                ///< Microseconds of time

      unsigned char  m_nDataLength;                  ///< Message length
      unsigned char  m_acDataBuffer[MAX_UNDEFINED_LENGTH]; ///< Data buffer
};




#ifndef USE_DEFAULT_PACKING
   // packing mode default
#pragma pack(pop)
#endif

#endif // #ifndef LUA_CANOPEN_MESSAGE_HPP










