// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate Lua-API
// File          : Lua-CANMessageList.hpp
// Author        : Stefan Welisch
// Copyright     : (C) 2009 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id:$
//
// $Log:$
//
// ----------------------------------------------------------------------------

#ifndef LUA_CANOPEN_MESSAGE_LIST_HPP
#define LUA_CANOPEN_MESSAGE_LIST_HPP

// Includes -------------------------------------------------------------------
#include <sys/types.h>
#include <vector>
#include <map>
#include "LuaCANOpenMessage.hpp"
#include "AnaMutex.h"

// Defines --------------------------------------------------------------------

class CLuaCANopenPDOList
{
   public:
      typedef std::vector<CLuaCANopenPDOMessage>  tvPDOMessage;
      typedef std::map<int, tvPDOMessage *> tMessageLists;

      /** Constructor. */
      CLuaCANopenPDOList( void );

      /** Destructor */
      virtual ~CLuaCANopenPDOList( void );

      void Push( const unsigned int nHandle, CLuaCANopenPDOMessage & oMessage );
      int Pop( const unsigned int nHandle, CLuaCANopenPDOMessage * pMessage );
      int Size( const unsigned int nHandle );

   protected:
      CAnaMutex     m_oMutex;
      tMessageLists m_UsedLists;
};


class CLuaCANopenSYNCList
{
   public:
      typedef std::vector<CLuaCANopenSYNCMessage>  tvSYNCMessage;
      typedef std::map<int, tvSYNCMessage *> tMessageLists;

      /** Constructor. */
      CLuaCANopenSYNCList( void );

      /** Destructor */
      virtual ~CLuaCANopenSYNCList( void );

      void Push( const unsigned int nHandle, CLuaCANopenSYNCMessage & oMessage );
      int Pop( const unsigned int nHandle, CLuaCANopenSYNCMessage * pMessage );
      int Size( const unsigned int nHandle );

   protected:
      CAnaMutex     m_oMutex;
      tMessageLists m_UsedLists;
};



class CLuaCANopenEMCYList
{
   public:
      typedef std::vector<CLuaCANopenEMCYMessage>  tvEMCYMessage;
      typedef std::map<int, tvEMCYMessage *> tMessageLists;

      /** Constructor. */
      CLuaCANopenEMCYList( void );

      /** Destructor */
      virtual ~CLuaCANopenEMCYList( void );

      void Push( const unsigned int nHandle, CLuaCANopenEMCYMessage & oMessage );
      int Pop( const unsigned int nHandle, CLuaCANopenEMCYMessage * pMessage );
      int Size( const unsigned int nHandle );

   protected:
      CAnaMutex     m_oMutex;
      tMessageLists m_UsedLists;
};



class CLuaCANopenGUARDList
{
   public:
      typedef std::vector<CLuaCANopenGUARDMessage>  tvGUARDMessage;
      typedef std::map<int, tvGUARDMessage *> tMessageLists;

      /** Constructor. */
      CLuaCANopenGUARDList( void );

      /** Destructor */
      virtual ~CLuaCANopenGUARDList( void );

      void Push( const unsigned int nHandle, CLuaCANopenGUARDMessage & oMessage );
      int Pop( const unsigned int nHandle, CLuaCANopenGUARDMessage * pMessage );
      int Size( const unsigned int nHandle );

   protected:
      CAnaMutex     m_oMutex;
      tMessageLists m_UsedLists;
};




class CLuaCANopenUndefinedList
{
   public:
      typedef std::vector<CLuaCANopenUndefinedMessage>  tvUndefinedMessage;
      typedef std::map<int, tvUndefinedMessage *> tMessageLists;

      /** Constructor. */
      CLuaCANopenUndefinedList( void );

      /** Destructor */
      virtual ~CLuaCANopenUndefinedList( void );

      void Push( const unsigned int nHandle, CLuaCANopenUndefinedMessage & oMessage );
      int Pop( const unsigned int nHandle, CLuaCANopenUndefinedMessage * pMessage );
      int Size( const unsigned int nHandle );

   protected:
      CAnaMutex     m_oMutex;
      tMessageLists m_UsedLists;
};

#endif // #ifndef LUA_CANOPEN_MESSAGE_LIST_HPP










