// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate Lua-API
// File          : LuaTCPServerFunctions.h
// Author        : Stefan Welisch
// Copyright     : (C) 2009 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id:$
//
// $Log:$
//
// ----------------------------------------------------------------------------

// Includes -------------------------------------------------------------------
#include "AnaGateMsg.hpp"
#include <AnaTCPIPServerContainer.h>

/// Includes for Scripting
extern "C"
{
   #include "lua.h"
   #include "lauxlib.h"
   #include "lualib.h"

   extern lua_State * L;
}


namespace lua {
void RegisterTCPServerFunctions( lua_State * L );
};

class CTCPServerContainer : public CAnaTCPIPServerContainer
{
public:
   CTCPServerContainer(unsigned int nPort);

   virtual ~CTCPServerContainer();

   // ----------------------------------------------------------------------------
   /** Mit dieser Methode wird eine neue Verbindungsklasse erzeugt.
         @return Zeiger auf die neu erstellte Klasse
   */
   virtual CAnaTCPIPServerBase * CreateNewClass( int nID );

   CAnaTCPIPServerBase * GetConnection( int nID );

   std::vector<CAnaTCPIPServerBase *> GetAllConnections( void );

};


/** Beschreibung der Klasse CServerConnection */
class CServerConnection : public CAnaTCPIPServerBase
{
   public:
      /** Konstruktor
          @param poTCPIPServer Zeiger auf die Verwaltungsklasse
          @param nID           Indes der Verbindungsklasse
      */
      CServerConnection( CAnaTCPIPServerContainer * poTCPIPServer, const int nID );
      /** Destruktor */
      virtual ~CServerConnection();

      int GetAnaGateMessage( CAnaGateMsg ** pMsg );

   protected:
   private:
      /** Wird aufgerufen, sobald Daten empfangen wurden.
          @param pData      emfangene Daten
          @param nDataLen   Lnge (in Bytes) der empfangenen Daten
      */
      void DataInd( const unsigned char * pData, const int nDataLen );

      /** */
      void ConInd( const int nID, bool bConnected );

      std::vector<CAnaGateMsg *> m_vMessageList;
};


int LS_TCPServerStart( lua_State * L );
int LS_TCPServerStop( lua_State * L );
int LS_TCPServerGetMessage( lua_State * L );
int LS_TCPServerSendMessage( lua_State * L );
int LS_TCPServerGetConnections( lua_State * L );
